'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ParseSessionFacade = exports.ParseSession = undefined;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.makeParseSession = makeParseSession;

var _CoreManager = require('./CoreManager');

var _isRevocableSession = require('./isRevocableSession');

var _isRevocableSession2 = _interopRequireDefault(_isRevocableSession);

var _ParseObject2 = require('./ParseObject');

var _ParseObject3 = _interopRequireDefault(_ParseObject2);

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

var _ParseUser = require('./ParseUser');

var _ParseUser2 = _interopRequireDefault(_ParseUser);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

function makeParseSession(coreManager) {
  return function (_ParseSession) {
    (0, _inherits3.default)(_class, _ParseSession);

    function _class(attributes) {
      (0, _classCallCheck3.default)(this, _class);
      return (0, _possibleConstructorReturn3.default)(this, (_class.__proto__ || (0, _getPrototypeOf2.default)(_class)).call(this, attributes, coreManager));
    }

    return _class;
  }(ParseSession);
}

/**
 * <p>A Parse.Session object is a local representation of a revocable session.
 * This class is a subclass of a Parse.Object, and retains the same
 * functionality of a Parse.Object.</p>
 * @alias Parse.Session
 * @extends Parse.Object
 */
/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

var ParseSession = exports.ParseSession = function (_ParseObject) {
  (0, _inherits3.default)(ParseSession, _ParseObject);

  /**
   * 
   * @param {Object} attributes The initial set of data to store in the user.
   */
  function ParseSession(attributes, coreManager) {
    (0, _classCallCheck3.default)(this, ParseSession);

    var _this2 = (0, _possibleConstructorReturn3.default)(this, (ParseSession.__proto__ || (0, _getPrototypeOf2.default)(ParseSession)).call(this, '_Session', undefined, undefined, coreManager));

    if (attributes && (typeof attributes === 'undefined' ? 'undefined' : (0, _typeof3.default)(attributes)) === 'object') {
      if (!_this2.set(attributes || {})) {
        throw new Error('Can\'t create an invalid Session');
      }
    }
    return _this2;
  }

  (0, _createClass3.default)(ParseSession, [{
    key: 'getSessionToken',

    /**
     * Returns the session token string.
      * @return {String}
     */
    value: function () {
      var token = this.get('sessionToken');
      if (typeof token === 'string') {
        return token;
      }
      return '';
    }
  }], [{
    key: 'readOnlyAttributes',
    value: function () {
      return ['createdWith', 'expiresAt', 'installationId', 'restricted', 'sessionToken', 'user'];
    }
  }]);
  return ParseSession;
}(_ParseObject3.default);

var ParseSessionFacade = exports.ParseSessionFacade = function () {
  function ParseSessionFacade(coreManager) {
    (0, _classCallCheck3.default)(this, ParseSessionFacade);

    if (coreManager.getSessionFacade()) {
      throw new Error('Cannot set ParseSessionFacade mutliple times to coreManager');
    }
    this._CoreManager = coreManager;
    this._CoreManager.setSessionFacade(this);
    this.Instance = makeParseSession(this._CoreManager);

    this._CoreManager.setSessionController(new DefaultController(this._CoreManager));
    this._CoreManager.getObjectFacade().registerSubclass('_Session', this.Instance);
  }

  /**
   * Retrieves the Session object for the currently logged in session.
    * @static
   * @return {Parse.Promise} A promise that is resolved with the Parse.Session
   *   object after it has been fetched. If there is no current user, the
   *   promise will be rejected.
   */

  (0, _createClass3.default)(ParseSessionFacade, [{
    key: 'current',
    value: function (options) {
      options = options || {};
      var controller = this._CoreManager.getSessionController();

      var sessionOptions = {};
      if (options.hasOwnProperty('useMasterKey')) {
        sessionOptions.useMasterKey = options.useMasterKey;
      }
      return this._CoreManager.getUserFacade().currentAsync().then(function (user) {
        if (!user) {
          return _ParsePromise2.default.error('There is no current user.');
        }
        user.getSessionToken();

        sessionOptions.sessionToken = user.getSessionToken();
        return controller.getSession(sessionOptions);
      });
    }

    /**
     * Determines whether the current session token is revocable.
     * This method is useful for migrating Express.js or Node.js web apps to
     * use revocable sessions. If you are migrating an app that uses the Parse
     * SDK in the browser only, please use Parse.User.enableRevocableSession()
     * instead, so that sessions can be automatically upgraded.
      * @static
     * @return {Boolean}
     */

  }, {
    key: 'isCurrentSessionRevocable',
    value: function () {
      var currentUser = this._CoreManager.getUserFacade().current();
      if (currentUser) {
        return (0, _isRevocableSession2.default)(currentUser.getSessionToken() || '');
      }
      return false;
    }
  }]);
  return ParseSessionFacade;
}();

var DefaultController = function () {
  function DefaultController(coreManager) {
    (0, _classCallCheck3.default)(this, DefaultController);

    this._CoreManager = coreManager;
  }

  (0, _createClass3.default)(DefaultController, [{
    key: 'getSession',
    value: function (options) {
      var RESTController = this._CoreManager.getRESTController();
      var session = new ParseSession(undefined, this._CoreManager);

      return RESTController.request('GET', 'sessions/me', {}, options).then(function (sessionData) {
        session._finishFetch(sessionData);
        session._setExisted(true);
        return session;
      });
    }
  }]);
  return DefaultController;
}();

var _ParseSessionFacade = new ParseSessionFacade(_CoreManager.CoreManager);

exports.default = ParseSession;