'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _CoreManager = require('./CoreManager');

var _decode = require('./decode');

var _decode2 = _interopRequireDefault(_decode);

var _encode = require('./encode');

var _encode2 = _interopRequireDefault(_encode);

var _ParseError = require('./ParseError');

var _ParseError2 = _interopRequireDefault(_ParseError);

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/**
 * Contains functions for calling and declaring
 * <a href="/docs/cloud_code_guide#functions">cloud functions</a>.
 * <p><strong><em>
 *   Some functions are only available from Cloud Code.
 * </em></strong></p>
 *
 * @class Parse.Cloud
 * @static
 * @hideconstructor
 */

/**
 * Makes a call to a cloud function.
 * @method run
 * @name Parse.Cloud.run
 * @param {String} name The function name.
 * @param {Object} data The parameters to send to the cloud function.
 * @param {Object} options A Backbone-style options object
 * options.success, if set, should be a function to handle a successful
 * call to a cloud function.  options.error should be a function that
 * handles an error running the cloud function.  Both functions are
 * optional.  Both functions take a single argument.
 * @return {Parse.Promise} A promise that will be resolved with the result
 * of the function.
 */

var ParseCloudFacade = function () {
  function ParseCloudFacade(coreManager) {
    (0, _classCallCheck3.default)(this, ParseCloudFacade);

    this._CoreManager = coreManager;
    this._CoreManager.setCloudFacade(this);
    this._CoreManager.setCloudController(new DefaultController(this._CoreManager));
  }

  (0, _createClass3.default)(ParseCloudFacade, [{
    key: 'run',
    value: function (name, data, options) {
      options = options || {};

      if (typeof name !== 'string' || name.length === 0) {
        throw new TypeError('Cloud function name must be a string.');
      }

      var requestOptions = {};
      if (options.useMasterKey) {
        requestOptions.useMasterKey = options.useMasterKey;
      }
      if (options.sessionToken) {
        requestOptions.sessionToken = options.sessionToken;
      }

      return this._CoreManager.getCloudController().run(name, data, requestOptions)._thenRunCallbacks(options);
    }
  }]);
  return ParseCloudFacade;
}(); /**
      * Copyright (c) 2015-present, Parse, LLC.
      * All rights reserved.
      *
      * This source code is licensed under the BSD-style license found in the
      * LICENSE file in the root directory of this source tree. An additional grant
      * of patent rights can be found in the PATENTS file in the same directory.
      *
      * 
      */

exports.default = ParseCloudFacade;

var DefaultController = function () {
  function DefaultController(coreManager) {
    (0, _classCallCheck3.default)(this, DefaultController);

    this._CoreManager = coreManager;
  }

  (0, _createClass3.default)(DefaultController, [{
    key: 'run',
    value: function (name, data, options) {
      var RESTController = this._CoreManager.getRESTController();

      var payload = (0, _encode2.default)(data, true);

      var requestOptions = {};
      if (options.hasOwnProperty('useMasterKey')) {
        requestOptions.useMasterKey = options.useMasterKey;
      }
      if (options.hasOwnProperty('sessionToken')) {
        requestOptions.sessionToken = options.sessionToken;
      }

      var request = RESTController.request('POST', 'functions/' + name, payload, requestOptions);

      return request.then(function (res) {
        var decoded = (0, _decode2.default)(res, this._CoreManager);
        if (decoded && decoded.hasOwnProperty('result')) {
          return _ParsePromise2.default.as(decoded.result);
        }
        return _ParsePromise2.default.error(new _ParseError2.default(_ParseError2.default.INVALID_JSON, 'The server returned an invalid response.'));
      })._thenRunCallbacks(options);
    }
  }]);
  return DefaultController;
}();

var _ParseCloudFacase = new ParseCloudFacade(_CoreManager.CoreManager);