'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FacebookUtilsFacade = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _parseDate = require('./parseDate');

var _parseDate2 = _interopRequireDefault(_parseDate);

var _CoreManager = require('./CoreManager');

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * -weak
 */

var provider = function () {
  function provider(facebookUtils) {
    (0, _classCallCheck3.default)(this, provider);

    this.facebookUtils = facebookUtils;
  }

  (0, _createClass3.default)(provider, [{
    key: 'authenticate',
    value: function (options) {
      var _this = this;

      if (typeof FB === 'undefined') {
        options.error(this, 'Facebook SDK not found.');
      }
      FB.login(function (response) {
        if (response.authResponse) {
          if (options.success) {
            options.success(_this, {
              id: response.authResponse.userID,
              access_token: response.authResponse.accessToken,
              expiration_date: new Date(response.authResponse.expiresIn * 1000 + new Date().getTime()).toJSON()
            });
          }
        } else {
          if (options.error) {
            options.error(_this, response);
          }
        }
      }, {
        scope: this.facebookUtils.requestedPermissions
      });
    }
  }, {
    key: 'restoreAuthentication',
    value: function (authData) {
      if (authData) {
        var expiration = (0, _parseDate2.default)(authData.expiration_date);
        var expiresIn = expiration ? (expiration.getTime() - new Date().getTime()) / 1000 : 0;

        var authResponse = {
          userID: authData.id,
          accessToken: authData.access_token,
          expiresIn: expiresIn
        };
        var newOptions = {};
        if (this.facebookUtils.initOptions) {
          for (var key in this.facebookUtils.initOptions) {
            newOptions[key] = this.facebookUtils.initOptions[key];
          }
        }
        newOptions.authResponse = authResponse;

        // Suppress checks for login status from the browser.
        newOptions.status = false;

        // If the user doesn't match the one known by the FB SDK, log out.
        // Most of the time, the users will match -- it's only in cases where
        // the FB SDK knows of a different user than the one being restored
        // from a Parse User that logged in with username/password.
        var existingResponse = FB.getAuthResponse();
        if (existingResponse && existingResponse.userID !== authResponse.userID) {
          FB.logout();
        }

        FB.init(newOptions);
      }
      return true;
    }
  }, {
    key: 'getAuthType',
    value: function () {
      return 'facebook';
    }
  }, {
    key: 'deauthenticate',
    value: function () {
      this.restoreAuthentication(null);
    }
  }]);
  return provider;
}();

/**
 * Provides a set of utilities for using Parse with Facebook.
 * @class Parse.FacebookUtils
 * @static
 * @hideconstructor
 */

var FacebookUtilsFacade = exports.FacebookUtilsFacade = function () {
  function FacebookUtilsFacade(coreManager) {
    (0, _classCallCheck3.default)(this, FacebookUtilsFacade);

    this._CoreManager = coreManager;
    this.PUBLIC_KEY = "*";
    this.initialized = false;
    this.requestedPermissions;
    this.initOptions;
  }

  /**
   * Initializes Parse Facebook integration.  Call this function after you
   * have loaded the Facebook Javascript SDK with the same parameters
   * as you would pass to<code>
   * <a href=
   * "https://developers.facebook.com/docs/reference/javascript/FB.init/">
   * FB.init()</a></code>.  Parse.FacebookUtils will invoke FB.init() for you
   * with these arguments.
   *
   * @method init
   * @name Parse.FacebookUtils.init
   * @param {Object} options Facebook options argument as described here:
   *   <a href=
   *   "https://developers.facebook.com/docs/reference/javascript/FB.init/">
   *   FB.init()</a>. The status flag will be coerced to 'false' because it
   *   interferes with Parse Facebook integration. Call FB.getLoginStatus()
   *   explicitly if this behavior is required by your application.
   */

  (0, _createClass3.default)(FacebookUtilsFacade, [{
    key: 'init',
    value: function (options) {
      if (typeof FB === 'undefined') {
        throw new Error('The Facebook JavaScript SDK must be loaded before calling init.');
      }
      this.initOptions = {};
      if (options) {
        for (var key in options) {
          this.initOptions[key] = options[key];
        }
      }
      if (this.initOptions.status && typeof console !== 'undefined') {
        var warn = console.warn || console.log || function () {};
        warn.call(console, 'The "status" flag passed into' + ' FB.init, when set to true, can interfere with Parse Facebook' + ' integration, so it has been suppressed. Please call' + ' FB.getLoginStatus() explicitly if you require this behavior.');
      }
      this.initOptions.status = false;
      FB.init(this.initOptions);
      this._CoreManager.getUserFacade()._registerAuthenticationProvider(provider);
      this.initialized = true;
    }

    /**
     * Gets whether the user has their account linked to Facebook.
     *
     * @method isLinked
     * @name Parse.FacebookUtils.isLinked
     * @param {Parse.User} user User to check for a facebook link.
     *     The user must be logged in on this device.
     * @return {Boolean} <code>true</code> if the user has their account
     *     linked to Facebook.
     */

  }, {
    key: 'isLinked',
    value: function (user) {
      return user._isLinked('facebook');
    }

    /**
     * Logs in a user using Facebook. This method delegates to the Facebook
     * SDK to authenticate the user, and then automatically logs in (or
     * creates, in the case where it is a new user) a Parse.User.
     *
     * @method logIn
     * @name Parse.FacebookUtils.logIn
     * @param {(String|Object)} permissions The permissions required for Facebook
     *    log in.  This is a comma-separated string of permissions.
     *    Alternatively, supply a Facebook authData object as described in our
     *    REST API docs if you want to handle getting facebook auth tokens
     *    yourself.
     * @param {Object} options Standard options object with success and error
     *    callbacks.
     * @returns {Parse.Promise}
     */

  }, {
    key: 'logIn',
    value: function (permissions, options) {
      if (!permissions || typeof permissions === 'string') {
        if (!this.initialized) {
          throw new Error('You must initialize FacebookUtils before calling logIn.');
        }
        this.requestedPermissions = permissions;
        return this._CoreManager.getUserFacade()._logInWith('facebook', options);
      } else {
        var newOptions = {};
        if (options) {
          for (var key in options) {
            newOptions[key] = options[key];
          }
        }
        newOptions.authData = permissions;
        return this._CoreManager.getUserFacade()._logInWith('facebook', newOptions);
      }
    }

    /**
     * Links Facebook to an existing PFUser. This method delegates to the
     * Facebook SDK to authenticate the user, and then automatically links
     * the account to the Parse.User.
     *
     * @method link
     * @name Parse.FacebookUtils.link
     * @param {Parse.User} user User to link to Facebook. This must be the
     *     current user.
     * @param {(String|Object)} permissions The permissions required for Facebook
     *    log in.  This is a comma-separated string of permissions.
     *    Alternatively, supply a Facebook authData object as described in our
     *    REST API docs if you want to handle getting facebook auth tokens
     *    yourself.
     * @param {Object} options Standard options object with success and error
     *    callbacks.
     * @returns {Parse.Promise}
     */

  }, {
    key: 'link',
    value: function (user, permissions, options) {
      if (!permissions || typeof permissions === 'string') {
        if (!this.initialized) {
          throw new Error('You must initialize FacebookUtils before calling link.');
        }
        this.requestedPermissions = permissions;
        return user._linkWith('facebook', options);
      } else {
        var newOptions = {};
        if (options) {
          for (var key in options) {
            newOptions[key] = options[key];
          }
        }
        newOptions.authData = permissions;
        return user._linkWith('facebook', newOptions);
      }
    }

    /**
     * Unlinks the Parse.User from a Facebook account.
     *
     * @method unlink
     * @name Parse.FacebookUtils.unlink
     * @param {Parse.User} user User to unlink from Facebook. This must be the
     *     current user.
     * @param {Object} options Standard options object with success and error
     *    callbacks.
     * @returns {Parse.Promise}
     */

  }, {
    key: 'unlink',
    value: function (user, options) {
      if (!this.initialized) {
        throw new Error('You must initialize FacebookUtils before calling unlink.');
      }
      return user._unlinkFrom('facebook', options);
    }
  }]);
  return FacebookUtilsFacade;
}();