'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Storage = exports.StorageFacade = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _CoreManager = require('./CoreManager');

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

var StorageFacade = exports.StorageFacade = function () {
  function StorageFacade(coreManager) {
    (0, _classCallCheck3.default)(this, StorageFacade);

    this._CoreManager = coreManager;
    this._CoreManager.setStorageFacade(this);

    this._CoreManager.setStorageController(require('./StorageController.default'));
  }

  (0, _createClass3.default)(StorageFacade, [{
    key: 'async',
    value: function () {
      var controller = this._CoreManager.getStorageController();
      return !!controller.async;
    }
  }, {
    key: 'getItem',
    value: function (path) {
      var controller = this._CoreManager.getStorageController();
      if (controller.async === 1) {
        throw new Error('Synchronous storage is not supported by the current storage controller');
      }
      return controller.getItem(path);
    }
  }, {
    key: 'getItemAsync',
    value: function (path) {
      var controller = this._CoreManager.getStorageController();
      if (controller.async === 1) {
        return controller.getItemAsync(path);
      }
      return _ParsePromise2.default.as(controller.getItem(path));
    }
  }, {
    key: 'setItem',
    value: function (path, value) {
      var controller = this._CoreManager.getStorageController();
      if (controller.async === 1) {
        throw new Error('Synchronous storage is not supported by the current storage controller');
      }
      return controller.setItem(path, value);
    }
  }, {
    key: 'setItemAsync',
    value: function (path, value) {
      var controller = this._CoreManager.getStorageController();
      if (controller.async === 1) {
        return controller.setItemAsync(path, value);
      }
      return _ParsePromise2.default.as(controller.setItem(path, value));
    }
  }, {
    key: 'removeItem',
    value: function (path) {
      var controller = this._CoreManager.getStorageController();
      if (controller.async === 1) {
        throw new Error('Synchronous storage is not supported by the current storage controller');
      }
      return controller.removeItem(path);
    }
  }, {
    key: 'removeItemAsync',
    value: function (path) {
      var controller = this._CoreManager.getStorageController();
      if (controller.async === 1) {
        return controller.removeItemAsync(path);
      }
      return _ParsePromise2.default.as(controller.removeItem(path));
    }
  }, {
    key: 'generatePath',
    value: function (path) {
      if (!this._CoreManager.get('APPLICATION_ID')) {
        throw new Error('You need to call Parse.initialize before using Parse.');
      }
      if (typeof path !== 'string') {
        throw new Error('Tried to get a Storage path that was not a String.');
      }
      if (path[0] === '/') {
        path = path.substr(1);
      }
      return 'Parse/' + this._CoreManager.get('APPLICATION_ID') + '/' + path;
    }
  }, {
    key: '_clear',
    value: function () {
      var controller = this._CoreManager.getStorageController();
      if (controller.hasOwnProperty('clear')) {
        controller.clear();
      }
    }
  }]);
  return StorageFacade;
}();

var Storage = exports.Storage = new StorageFacade(_CoreManager.CoreManager);

exports.default = Storage;